/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FullerScreen.
 *
 * The Initial Developer of the Original Code is
 * Disruptive Innovations SARL
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * Dumb function to resize canvas 
 * @param myCanvas: a canvas element
 * @param mode:
  * resize using either kGCResizePercent or kGCResizePixel
  * resize either:
   * kGCResizeNoRatio no aspect ratio respect, using both dimensions
   * kGCResizeRatioX aspect ratio preserved using the x dimension (kGCResizePercent or kGCResizePixel)
   * kGCResizeRatioY aspect ratio preserved using the y dimension (only valid for kGCResizePixel)
   * default is kGCResizeNoRatio & kGCResizePixel
  * example: (kGCResizePixel & kGCResizeRatioY) will resize using the given Y dimension in pixel and respect aspect ratio
 * @param newWidth: width (depends of the mode)
 * @param newHeight: height (depend of the mode)
 */

const kGCResizeNoRatio = 1;
const kGCResizeRatioX = 3;
const kGCResizeRatioY = 5;
const kGCResizePixel = 14;
const kGCResizePercent = 15;

var gCanvasResize = function gCanvasResize(myCanvas, mode, newWidth, newHeight)
{
  switch(mode)
  {
  // Aspect ratio preserved, using explicit width
    case (kGCResizePixel & kGCResizeRatioX):
      newHeight = Math.round(myCanvas.height/myCanvas.width*newWidth);
    break;
  // Aspect ratio preserved, using explicit height
    case (kGCResizePixel & kGCResizeRatioY):
      newWidth = Math.round(myCanvas.width/myCanvas.height*newHeight);
    break;
  // Aspect ratio discarded, using percent redimensioning
    case (kGCResizePercent & kGCResizeNoRatio):
      newWidth = Math.round(myCanvas.width*newWidth/100);
      newHeight = Math.round(myCanvas.height*newHeight/100);
    break;
  // Aspect ratio preserver, using ratio redimensioning
    case ((kGCResizePercent & kGCResizeRatioX) || (kGCResizePercent & kGCResizeRatioY)):
      newWidth = Math.round(myCanvas.width*newWidth/100);
      newHeight = Math.round(myCanvas.height*newWidth/100);
    break;
  // No aspect ratio. Resize to given explicit dimensions
    default:
    break;
  }
  myCanvas.style.width = newWidth + "px";
  myCanvas.style.height = newHeight + "px";
  myCanvas.width = newWidth;
  myCanvas.height = newHeight;
}

/**
 * Dumb function to specify canvas scaling
 * Given an original rectangle, decide of a scaling method on canvas
 * @param myCanvas: a canvas element
 * @param mode:
  * kGCScaleExpand: complete original rectangle will be scaled in, mangling aspect ratio if necessary
  * kGCScaleInside: original rectangle will be scaled in entirely, with aspect ratio preserved 
   (eg: part of the canvas will be left blank)
  * kGCScaleOutside: original rectangle will be scaled in, with aspect ratio preserved, as to fill the canvas
   (eg: part of the original will not be visible)
 * @param originalWidth: the width of the original rectangle in pixels
 * @param originalHeight: the height of the original rectangle in pixels
 */

const kGCScaleExpand = 0; // default
const kGCScaleInside = 1;
const kGCScaleOutside = 2;

var gCanvasScale = function gCanvasScale(myCanvas, mode, originalWidth, originalHeight)
{
  var scaleX = myCanvas.width/originalWidth;
  var scaleY = myCanvas.height/originalHeight;
  switch(mode)
  {
  // Aspect ratio preserved, maximum filling
    case (kGCScaleOutside):
      (scaleX < scaleY) ? scaleX = scaleY : scaleY = scaleX;
    break;
  // Aspect ratio preserved, fit in entirely
    case (kGCScaleInside):
      (scaleX > scaleY) ? scaleX = scaleY : scaleY = scaleX;
    break;
    default:
    break;
  }
  myCanvas.getContext("2d").scale(scaleX, scaleY);
}

/**
 * Sauvegarde du canvas
 */

var gCanvasSaveFile = function gCanvasSaveFile(canvas, destFile) {
  // convertit le chemin texte en un nsIFile
/*  var file = Components.classes["@mozilla.org/file/local;1"]
                       .createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(destFile);*/
  var file = destFile;

  // cre une url de donnes depuis le canvas et cre alors les URIs de la source et les cibles
  var io = Components.classes["@mozilla.org/network/io-service;1"]
                     .getService(Components.interfaces.nsIIOService);
  var source = io.newURI(canvas.toDataURL("image/png", ""), "UTF8", null);
  var target = io.newFileURI(file)
  
  // prpare  sauvegarder les donnes canvas
  var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
                          .createInstance(Components.interfaces.nsIWebBrowserPersist);
  
  persist.persistFlags = Components.interfaces.nsIWebBrowserPersist.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
  persist.persistFlags |= Components.interfaces.nsIWebBrowserPersist.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;
  
  // affiche un dialogue de tlchargement (supprimez ces 3 lignes pour un tlchargement silencieux)
  //var xfer = Components.classes["@mozilla.org/transfer;1"]
  //                     .createInstance(Components.interfaces.nsITransfer);
  //xfer.init(source, target, "", null, null, null, persist);
  //persist.progressListener = xfer;
  
  // sauvegarde les donnes canvas vers le fichier
  persist.saveURI(source, null, null, null, null, file);
}



/**
 * TODO
 * Private function to perform a copy after the canvas has been sized and scaled
 * For copies preserving aspect ratios:
  * for insideCopies, the result may be aligned to left, center, right (resp: top center bottom)
  * for outsideCopies, the origin may be taken from left, center, right (resp: top center bottom)
 */

function gCanvasCopy(myCanvas, mode, contentWindow, x, y, w, h, transparent)
{
  var color = ((typeof transparent != 'undefined') ? "rgb(0,0,0,0)" : "rgb(0,0,0)");
  myCanvas.getContext("2d").drawWindow(contentWindow, x, y, w, h, color);
}



function gCanvasPosition(nsIElement) {
  var startX = 0;
  var startY = 0;
  while (nsIElement)
  {
    startX += nsIElement.offsetLeft;
    startY += nsIElement.offsetTop;
    nsIElement = nsIElement.offsetParent;
  }
  return [startX, startY];
}


