/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FullerScreen.
 *
 * The Initial Developer of the Original Code is
 * Disruptive Innovations SARL
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *
 * ***** END LICENSE BLOCK ***** */

function KeyUp(aEvent) {
  switch (aEvent.which) {
    case 32: // spacebar
    case 34: // page down
    case 39: // rightkey
    case 40: // downkey
      ChangeSlide(+1);
      break;
    case 33: // page up
    case 37: // leftkey
    case 38: // upkey
      ChangeSlide(-1);
      break;
    case 36: // home
      GoToSlide(0);
      break;
    case 35: // end
      GoToSlide(-1);
      break;
    case 122: // F11
      if (aEvent.shiftKey)
        FS_slideManager.toggleSlideManager();
      break;
    default:
      break;
  }
  return false;
}

function Clicker(aEvent)
{
  if (!aEvent.button)
  {
    var originator = aEvent.originalTarget.getAttribute("anonid");
    var win = GetCurrentWindow();
    var projdata = win.FS_projection_data;
    if (!projdata)
      return;
    switch (originator)
    {
      case "_FSjs21":
        if (win.FS_projection_data.inSlideShowMode)
          ChangeSlide(-1);
        break;
      case "_FSjs23": 
        if (win.FS_projection_data.inSlideShowMode)
          ChangeSlide(+1);
        break;

      case "_FSjs12": 
        if (win.FS_projection_data.inSlideShowMode)
          GoToSlide(0);
        break;
      case "_FSjs32": 
        if (win.FS_projection_data.inSlideShowMode)
          GoToSlide(-1);
        break;

      case "_FSjs22": 
        if (win.FS_projection_data.inSlideShowMode)
          FS_slideManager.toggleSlideManager();
        break;

      case "_FSjs42": BrowserFullScreen(); break;

      default: break;
    }
  }
}

function CheckIfProjection()
{
  var browser = window.getBrowser();
  var doc = browser.contentDocument;
  var win = browser.contentWindow;
  var projdata = win.FS_projection_data;
  if (!projdata)
  {
    projdata = {
      inProjectionMode: false,
      inSlideShowMode: false,
      slideSelector: "",
      titleSelector: "",
      stepSelector: "",
      slides: null,
      titles: [],
      styles: [],
      slideStyles: [],
      steps: [],
      currentStep: -1,
      currentSlide: 0,

      slideManager: null
    };
    win.FS_projection_data = projdata;
    projdata = win.FS_projection_data;
  }

  if (!projdata.inProjectionMode)
  {
    var ss = doc.styleSheets;
    var tweaked = false;
    for (var i = 0; i < ss.length; i++)
    {
      tweaked |= FS_TweakStyleSheet(win, ss[i], ss[i], null);
    }
    if (tweaked)
      projdata.inProjectionMode = true;
  }

  var metas = doc.documentElement.getElementsByTagName("meta");
  var isSlideShow = false;
  for (var i = 0; i < metas.length; i++)
  {
    var metaContent = metas[i].getAttribute("content");
    switch (metas[i].getAttribute("name"))
    {
      case "slideselector":
        isSlideShow = true;
        projdata.slideSelector = metaContent;
        break;
      case "titleselector":
        projdata.titleSelector = metaContent;
        break;
      case "stepselector":
        projdata.stepSelector = metaContent;
      default:
        break;
    }
  }

  AddJoystick(doc, isSlideShow);

  if (!isSlideShow)
  {
    projdata.inSlideShowMode = false;
    return false;
  }

  if (projdata.inSlideShowMode)
  {
    FS_slideManager.fillSlidesMenu(win);
    return true;
  }

  // extracting strings from the locale file
  var gfullerscreenBundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
                             .getService(Components.interfaces.nsIStringBundleService);
  var mystrings = gfullerscreenBundle.createBundle(FS_data.kSTRINGBUNDLE);
  var FSslideshowmode = mystrings.GetStringFromName("FSslideshowmode");
  var FSswitch = mystrings.GetStringFromName("FSswitch");

  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                      .getService(Components.interfaces.nsIPromptService);
  var res = promptService.confirm(window,
                                  FSslideshowmode,
                                  FSswitch);
  if (!res)
    return true;

  projdata.inSlideShowMode = true;
  projdata.slides = getElementsBySelector(doc.documentElement, projdata.slideSelector, false);
  PreserveSlideStyles(win);
  doc.documentElement.addEventListener("keyup", KeyUp, false);
  setTimeout("FS_slideManager.fillSlidesMenu(null)", 50);
  return true;
}

function PreserveSlideStyles(aWindow)
{
  var win = aWindow ? aWindow : GetCurrentWindow();
  var slides = win.FS_projection_data.slides;
  if (slides && slides.length)
  {
    for (var i = 0; i < slides.length ; i++)
    {
      win.FS_projection_data.slideStyles.push(slides[i].style.display);
    }
  }
}

function GetCurrentWindow()
{
  var browser = window.getBrowser();
  return browser.contentWindow;
}

function ShowSlide(aSlideIndex, aWin, aIncrement)
{
  var win = aWin ? aWin : GetCurrentWindow();

  var projdata = win.FS_projection_data;

  if (!projdata)
    return;
  var slides = projdata.slides;
  if (slides && slides.length)
  {
    for (var i = 0; i < slides.length ; i++)
    {
      if (i == aSlideIndex || aSlideIndex == -1)
      {
        slides[i].style.display = projdata.slideStyles[i];
        if (aSlideIndex == -1)
        {
          var steps = getElementsBySelector(slides[i], "*", false);
          for (var j = 0; j < steps.length; j++)
          {
            var node = steps[j];
            node.style.visibility = "";
            if (node.className)
              node.className = node.className.replace( /__FSstep/g, "");
          }
        }
      }
      else
        slides[i].style.display = "none";
    }
    projdata.currentSlide = aSlideIndex;
    if (aSlideIndex != -1)
    {
      // we're really showing a slide
      var slide = slides[aSlideIndex];
      if (slide)
      {
        var browser = window.getBrowser();
        var doc = browser.contentDocument;
        projdata.steps = getElementsBySelector(slide, projdata.stepSelector, false);
        if (aIncrement == -1)
          projdata.currentStep = projdata.steps.length;
        else
          projdata.currentStep = -1;
        TryToChangeStep(aIncrement);
      }
    }
  }
}

function ChangeSlide(aIncrement)
{
  var win = GetCurrentWindow();
  var projdata = win.FS_projection_data;
  if (!projdata)
    return;
  if (TryToChangeStep(aIncrement))
    return;
  var currentSlide = projdata.currentSlide;
  var maxSlide     = projdata.slides.length;
  newSlide = currentSlide + aIncrement;
  if (newSlide < 0)
    newSlide = 0;
  if (newSlide >= maxSlide)
    newSlide = maxSlide - 1;
  // early way out if we can
 if (currentSlide == newSlide)
    return;
 ShowSlide(newSlide, win, aIncrement)
}

function GoToSlide(aSlideIndex)
{
  var win = GetCurrentWindow();
  var projdata = win.FS_projection_data;
  if (!projdata)
    return;
  var currentSlide = projdata.currentSlide;
  var maxSlide     = projdata.slides.length;
  if (aSlideIndex < 0 || aSlideIndex >= maxSlide)
    aSlideIndex = maxSlide - 1;
  // early way out if we can
 if (currentSlide == aSlideIndex)
    return;
 ShowSlide(aSlideIndex, win, +1)
}

function FS_TweakStyleSheet(win, o, sheet, importRule)
{
  var medialist = o.media;
  // this stylesheet has a media list
  // does it apply to projection ?
  var foundProjection = false;
  var foundScreen = false;
  var foundAll = false;
  for (var j = 0; j < medialist.length; j++)
  {
    var m = medialist.item(j);
    switch (m)
    {
      case "projection": foundProjection = true; break;
      case "screen":     foundScreen = true;     break;
      case "all":        foundAll= true;         break;
      default:           break;
    }
      
  }
  if (foundProjection)
  {
    if (!foundScreen && !foundAll)
    {
      // yes it does and it's not applied at all
      o.media.appendMedium("screen");
      // hack to trigger refresh
      if (importRule)
        importRule.media.appendMedium("screen");
      if ("disabled" in sheet)
      {
        sheet.disabled = true;
        sheet.disabled = false;
      }
      win.FS_projection_data.styles.push( { cssObject:  o,
                                            sheet:      sheet,
                                            importRule: importRule } );
      return true;
    }
  }
  else // not a projection stylesheet
  {
    // let's browse all rules in the object
    var rules = o.cssRules;
    var tweaked = false;
    for (var k = 0; k < rules.length; k++)
    {
      var r = rules.item(k);
      switch (r.type)
      {
        case CSSRule.IMPORT_RULE:
          tweaked |= FS_TweakStyleSheet(win, r.styleSheet, r.styleSheet, r);
          break;
        case CSSRule.MEDIA_RULE:
          tweaked |= FS_TweakStyleSheet(win, r, sheet, null);
          break;
        default:
          break;
      }
    }
    return tweaked;
  }
  return false;
}

function FS_ResetToScreen()
{
  var tabbrowser = document.getElementById("content");
  var browsers   = tabbrowser.browsers;
  for (var browserIndex = 0 ; browserIndex < browsers.length; browserIndex++)
  {
    var win = browsers[browserIndex].contentWindow;
    var projdata = win.FS_projection_data;

    if (!projdata)
      continue;

    var doc = win.document;
    var joystick = doc.getElementById("_FSjs");
    if (joystick)
    {
      try {
        joystick.removeEventListener("click", Clicker, false);
        var p = joystick.parentNode;
        p.removeChild(joystick);
      }
      catch(e) { alert(e); }
    }

    // early way out if we can
    if (!projdata.inProjectionMode)
      continue;

    for (var i = 0; i < projdata.styles.length; i++)
    {
      var fsi = projdata.styles[i];
      var medialist = fsi.cssObject.media;
      medialist.deleteMedium("screen");
      if (fsi.importRule)
        fsi.importRule.media.deleteMedium("screen");
      if (fsi.sheet && ("disabled" in fsi.sheet))
      {
        fsi.sheet.disabled = true;
        fsi.sheet.disabled = false;
      }
    }
    projdata.inProjectionMode = false;
    projdata.styles = [];

    // if we're in slideshow mode
    if (projdata.inSlideShowMode)
    {
      ShowSlide(-1, win, 1);
      projdata.slides = [];
      projdata.titles = [];
      projdata.slideStyles = [];
      var doc = browsers[browserIndex].contentDocument;
      if (projdata.slideManager)
        projdata.slideManager.parentNode.removeChild(
          projdata.slideManager);
      projdata.slideManager = null;
      doc.documentElement.removeEventListener("keyup", KeyUp, false);
      projdata.inSlideShowMode = false;
    }
  }
}

function TryToChangeStep(aIncrement)
{
  var win = GetCurrentWindow();
  var projdata = win.FS_projection_data;
  if (!projdata)
    return false;
  if (!projdata.steps.length)
    return false;

  var currentSlideIndex = projdata.currentSlide;
  var currentSlide = projdata.slides[currentSlideIndex];
  var currentStep  = projdata.currentStep;
  newStep = currentStep + aIncrement;
  if (newStep < 0)
    return false;

  if (newStep >= projdata.steps.length)
  {
    // we have this weird case where there is content AFTER the
    // last step... this is an implicit step
    var node = projdata.steps[currentStep];
    var foundEltSibling = false;
    while (!foundEltSibling && node && node != currentSlide)
    {
      // does node has an element sibling after him ?
      var sibling = node.nextSibling;
      while (sibling)
      {
        if (sibling.nodeType == Node.ELEMENT_NODE)
        {
          foundEltSibling = true;
          break;
        }
        sibling = sibling.nextSibling;
      }
      node = node.parentNode;
    }
    var foundHiddenElt = false;
    if (foundEltSibling)
    {
      var steps = currentSlide.getElementsByTagName("*");
      for (var i = 0 ; i < steps.length; i++)
      {
        var node = steps[i];
        if (node.style.visibility == "hidden")
          foundHiddenElt = true;
        node.style.visibility = "visible";
        if (node.className)
          node.className = node.className.replace( /__FSstep/g, "");
      }
      return foundHiddenElt;
    }
    return false;
  }

  var newStepElt = projdata.steps[newStep];
  TraverseBackSubtree(currentSlide, newStepElt, false);
  newStepElt.className = newStepElt.className + " __FSstep";
  projdata.currentStep = newStep;
  return true;
}

function TraverseBackSubtree(aNode, aStep, aFoundStep)
{
  var foundStep = aFoundStep;
  if (!aNode)
    return false;

  var node = aNode.lastChild;
  while (node)
  {
    if (node.nodeType == Node.ELEMENT_NODE)
    {
      if (!foundStep && node == aStep)
        foundStep = true;
      node.style.visibility = foundStep ? "visible" : "hidden";
      if (node.className)
        node.className = node.className.replace( /__FSstep/g, "");
      foundStep = TraverseBackSubtree(node, aStep, foundStep);
    }
    node = node.previousSibling;
  }
  return foundStep;
}

function AddJoystick(aDoc, aInSlideShow)
{
  var body = aDoc.body;
  if (!body)
    return;
  if (aDoc.getElementById("_FSjs"))
    return;

  var joystick = aDoc.createElement("div");
  joystick.setAttribute("id", "_FSjs");
  joystick.setAttribute("style", '-moz-binding: url("chrome://fullerscreen/content/joystick.xml#joystick")');
  joystick.setAttribute("slideshow", aInSlideShow ? "yes" : "no");

  body.appendChild(joystick);
  joystick.addEventListener("click", Clicker, false);
}
